"""System."""
from __future__ import annotations

from dataclasses import dataclass
from typing import Any


@dataclass
class SensorsWindowsSensor:
    """Sensors Windows Sensor."""

    id: str
    name: str
    type: str
    value: float | int | str | None = None


@dataclass
class SensorsWindowsHardware:
    """Sensors Windows Hardware."""

    id: str
    name: str
    type: str
    subhardware: list[SensorsWindowsHardware]
    sensors: list[SensorsWindowsSensor]


@dataclass
class SensorsNVIDIAChipset:
    """Sensors NVIDIA Chipset."""

    id: int
    name: str
    flags: str
    vendor_id: int
    vendor_name: str


@dataclass
class SensorsNVIDIADisplay:
    """Sensors NVIDIA Display."""

    id: int
    name: str
    active: bool
    available: bool
    connected: bool
    dynamic: bool
    aspect_horizontal: int
    aspect_vertical: int
    brightness_current: int
    brightness_default: int
    brightness_max: int
    brightness_min: int
    color_depth: str
    connection_type: str
    pixel_clock: int
    refresh_rate: int
    resolution_horizontal: int
    resolution_vertical: int


@dataclass
class SensorsNVIDIADriver:
    """Sensors NVIDIA Driver."""

    branch_version: str
    interface_version: str
    version: int


@dataclass
class SensorsNVIDIAGPU:
    """Sensors NVIDIA GPU."""

    id: int
    name: str
    bios_oem_revision: int | None = None
    bios_revision: int | None = None
    bios_version: str | None = None
    current_fan_speed_level: int | None = None
    current_fan_speed_rpm: int | None = None
    driver_model: int | None = None
    memory_available: int | None = None
    memory_capacity: int | None = None
    memory_maker: str | None = None
    serial: str | None = None
    system_type: str | None = None
    type: str | None = None


@dataclass
class SensorsNVIDIA:
    """Sensors NVIDIA."""

    chipset: SensorsNVIDIAChipset
    displays: list[SensorsNVIDIADisplay]
    driver: SensorsNVIDIADriver
    gpus: list[SensorsNVIDIAGPU]


@dataclass
class SensorsWindows:
    """Sensors Windows."""

    hardware: list[SensorsWindowsHardware] | None = None
    nvidia: SensorsNVIDIA | None = None


@dataclass
class Sensors:
    """Sensors."""

    # TODO: Add fan model
    fans: Any | None = None
    # TODO: Add temperatures model
    temperatures: Any | None = None
    windows_sensors: SensorsWindows | None = None



"""Model constants."""
# Model
from .keyboard_key import KeyboardKey
from .keyboard_text import KeyboardText
from .media_directories import MediaDirectory
from .media_files import MediaFile, MediaFiles
from .modules import ModulesData
from .modules.battery import Battery
from .modules.cpu import CPU
from .modules.disks import Disks
from .modules.displays import Display
from .modules.gpus import GPU
from .modules.media import Media
from .modules.memory import Memory
from .modules.networks import Networks
from .modules.processes import Process
from .modules.sensors import Sensors
from .modules.system import System
from .notification import Notification
from .open_path import OpenPath
from .open_url import OpenUrl
from .response import Response

MODEL_BATTERY = "battery"
MODEL_CPU = "cpu"
MODEL_DATA = "data"
MODEL_DISKS = "disks"
MODEL_DISPLAYS = "displays"
MODEL_GENERIC = "generic"
MODEL_GPUS = "gpus"
MODEL_KEYBOARD_KEY = "keyboard_key"
MODEL_KEYBOARD_TEXT = "keyboard_text"
MODEL_MEDIA = "media"
MODEL_MEDIA_DIRECTORIES = "media_directories"
MODEL_MEDIA_FILE = "media_file"
MODEL_MEDIA_FILES = "media_files"
MODEL_MEMORY = "memory"
MODEL_NETWORKS = "networks"
MODEL_NOTIFICATION = "notification"
MODEL_OPEN_PATH = "open_path"
MODEL_OPEN_URL = "open_url"
MODEL_PROCESSES = "processes"
MODEL_RESPONSE = "response"
MODEL_SECRETS = "secrets"
MODEL_SENSORS = "sensors"
MODEL_SETTINGS = "settings"
MODEL_SYSTEM = "system"

MODEL_MAP = {
    MODEL_BATTERY: Battery,
    MODEL_CPU: CPU,
    MODEL_DATA: ModulesData,
    MODEL_DISKS: Disks,
    MODEL_DISPLAYS: Display, # Map to Display not list[Display] so it can be mapped
    MODEL_GPUS: GPU, # Map to GPU not list[GPU] so it can be mapped
    MODEL_KEYBOARD_KEY: KeyboardKey,
    MODEL_KEYBOARD_TEXT: KeyboardText,
    MODEL_MEDIA_DIRECTORIES: MediaDirectory, # Map to MediaDirectory not list[MediaDirectory] so it can be mapped
    MODEL_MEDIA_FILE: MediaFile,
    MODEL_MEDIA_FILES: MediaFiles,
    MODEL_MEDIA: Media,
    MODEL_MEMORY: Memory,
    MODEL_NETWORKS: Networks,
    MODEL_NOTIFICATION: Notification,
    MODEL_OPEN_PATH: OpenPath,
    MODEL_OPEN_URL: OpenUrl,
    MODEL_PROCESSES: Process, # Map to Process not list[Process] so it can be mapped
    MODEL_RESPONSE: Response,
    MODEL_SENSORS: Sensors,
    MODEL_SYSTEM: System,
}



"""Notification."""
from __future__ import annotations

from dataclasses import dataclass
from typing import Any


@dataclass
class Action:
    """Notification Action."""

    command: str
    label: str
    data: dict[str, Any] | None = None


@dataclass
class Audio:
    """Notification Audio."""

    source: str
    volume: float | None = None


@dataclass
class Notification:
    """Notification."""

    title: str
    message: str | None = None
    icon: str | None = None
    image: str | None = None
    actions: list[Action] | None = None
    timeout: float | None = None
    audio: Audio | None = None



"""Media Files."""
from __future__ import annotations

from dataclasses import dataclass


@dataclass
class MediaFile:
    """Media File."""

    name: str
    path: str
    fullpath: str
    size: int
    last_accessed: float
    created: float
    modified: float
    is_directory: bool
    is_file: bool
    is_link: bool
    mime_type: str | None = None


@dataclass
class MediaFiles:
    """Media Files."""

    files: list[MediaFile]
    path: str



"""Networks."""
from __future__ import annotations

from dataclasses import dataclass


@dataclass
class NetworkAddress:
    """Network Address."""

    address: str | None = None
    family: str | None = None
    netmask: str | None = None
    broadcast: str | None = None
    ptp: str | None = None


@dataclass
class NetworkStats:
    """Network Stats."""

    isup: bool | None = None
    duplex: str | None = None
    speed: int | None = None
    mtu: int | None = None
    flags: list[str] | None = None


@dataclass
class NetworkConnection:
    """Network Connection."""

    fd: int | None = None
    family: int | None = None
    type: int | None = None
    laddr: str | None = None
    raddr: str | None = None
    status: str | None = None
    pid: int | None = None


@dataclass
class NetworkIO:
    """Network IO."""

    bytes_sent: int | None = None
    bytes_recv: int | None = None
    packets_sent: int | None = None
    packets_recv: int | None = None
    errin: int | None = None
    errout: int | None = None
    dropin: int | None = None
    dropout: int | None = None


@dataclass
class Network:
    """Network."""

    name: str | None = None
    addresses: list[NetworkAddress] | None = None
    stats: NetworkStats | None = None


@dataclass
class Networks:
    """Networks."""

    connections: list[NetworkConnection] | None = None
    io: NetworkIO | None = None
    networks: list[Network] | None = None
